@@ -28,7 +28,7 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
+ *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
  *   not derivative works.)                                                * 
@@ -63,7 +63,7 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -95,13 +95,13 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
- * with Nmap.                                                              *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: output.cc 6858 2008-02-28 18:52:06Z fyodor $ */
+/* $Id: output.cc 9744 2008-08-26 17:40:19Z david $ */
 
 #include "output.h"
 #include "osscan.h"
@@ -123,14 +123,15 @@ namespace std {};
 using namespace std;
 
 extern NmapOps o;
-static char *logtypes[LOG_NUM_FILES]=LOG_NAMES;
+static const char *logtypes[LOG_NUM_FILES]=LOG_NAMES;
 
 /* Used in creating skript kiddie style output.  |<-R4d! */
 static void skid_output(char *s)
 {
   int i;
-  for (i=0;s[i];i++)
-    if (rand()%2==0)
+  for (i=0; s[i]; i++)
+    /* We need a 50/50 chance here, use a random number */
+    if ((get_random_u8() & 0x01) == 0)
       /* Substitutions commented out are not known to me, but maybe look nice */
       switch(s[i])
 	{
@@ -142,7 +143,7 @@ static void skid_output(char *s)
 	case 'e':
 	case 'E': s[i]='3'; break;
 	case 'i':
-	case 'I': s[i]="!|1"[rand()%3]; break;
+	case 'I': s[i]="!|1"[get_random_u8() % 3]; break;
 	  /*      case 'k': s[i]='c'; break;
 	        case 'K': s[i]='C'; break;*/
 	case 'o':
@@ -158,8 +159,13 @@ static void skid_output(char *s)
 	}  
     else
       {
-	if (s[i]>='A' && s[i]<='Z' && (rand()%3==0)) s[i]+='a'-'A';
-	else if (s[i]>='a' && s[i]<='z' && (rand()%3==0)) s[i]-='a'-'A';
+	if (s[i] >= 'A' && s[i] <= 'Z' &&
+	    (get_random_u8() % 3 == 0)) {
+	  s[i] += 'a'-'A'; /* 1/3 chance of lower-case */
+	}
+	else if (s[i] >= 'a' && s[i] <= 'z' && (get_random_u8() % 3 == 0)) {
+	  s[i] -= 'a'-'A'; /* 1/3 chance of upper-case */
+	}
       }
 }
 
@@ -168,9 +174,8 @@ static char* xml_sf_convert (const char* str) {
   char *temp = (char *) safe_malloc(strlen(str) + 1);
   char *dst = temp, *src = (char *)str;
   char *ampptr = 0;
-  int charcount = 0;
 
-  while(*src && charcount < 2035) { /* 2048 - 14 */
+  while(*src) {
     if (strncmp(src, "\nSF:", 4) == 0) {
       src += 4;
       continue;
@@ -183,7 +188,6 @@ static char* xml_sf_convert (const char* str) {
       ampptr = 0;
     }
     *dst++ = *src++;
-    charcount++;
   }
   if (ampptr != 0) {
     *ampptr = '\0';
@@ -196,20 +200,22 @@ static char* xml_sf_convert (const char* str) {
 
 
 // Creates an XML <service> element for the information given in
-// serviceDeduction.  It will be 0-length if none is neccessary.
-// returns 0 for success.
-static int getServiceXMLBuf(struct serviceDeductions *sd, char *xmlbuf, 
-		     unsigned int xmlbuflen) {
+// serviceDeduction.  This function should only be called if ether
+// the service name or the service fingerprint is non-null.
+// Returns a pointer to a buffer containing the element,
+// you will have to call free on it.
+static char * getServiceXMLBuf(struct serviceDeductions *sd) {
   string versionxmlstring = "";
   char rpcbuf[128];
+  char confBuf[20];
   char *xml_product = NULL, *xml_version = NULL, *xml_extrainfo = NULL;
   char *xml_hostname = NULL, *xml_ostype = NULL, *xml_devicetype = NULL;
   char *xml_servicefp = NULL, *xml_servicefp_temp = NULL;
 
-  if (xmlbuflen < 1) return -1;
-  xmlbuf[0] = '\0';  
-  if (!sd->name && !sd->service_fp) return 0;
 
+  versionxmlstring =  "<service name=\"";
+  versionxmlstring += sd->name? sd->name : "unknown";
+  versionxmlstring += "\"";
   if (sd->product) {
     xml_product = xml_convert(sd->product);
     versionxmlstring += " product=\"";
@@ -274,25 +280,89 @@ static int getServiceXMLBuf(struct serviceDeductions *sd, char *xmlbuf,
 	     sd->rpc_program, sd->rpc_lowver, sd->rpc_highver);
   } else rpcbuf[0] = '\0';
 
-  Snprintf(xmlbuf, xmlbuflen, 
-	   "<service name=\"%s\"%s %smethod=\"%s\" conf=\"%d\"%s />", 
-	   sd->name? sd->name : "unknown",
-	   versionxmlstring.c_str(),
-	   (sd->service_tunnel == SERVICE_TUNNEL_SSL)? "tunnel=\"ssl\" " : "",
-	   (sd->dtype == SERVICE_DETECTION_TABLE)? "table" : "probed", 
-
-	   sd->name_confidence, rpcbuf);
-
-  return 0;
+  versionxmlstring += " ";
+  versionxmlstring += (sd->service_tunnel == SERVICE_TUNNEL_SSL)? "tunnel=\"ssl\" " : "";
+  versionxmlstring += "method=\"";
+  versionxmlstring += (sd->dtype == SERVICE_DETECTION_TABLE)? "table" : "probed";
+  versionxmlstring += "\" conf=\"";
+  Snprintf(confBuf,20,"%i",sd->name_confidence);
+  versionxmlstring += confBuf;
+  versionxmlstring += "\"";
+  versionxmlstring += rpcbuf;
+  versionxmlstring += " />";
+  return strdup(versionxmlstring.c_str());
 }
 
+#ifdef WIN32
+/* From tcpip.cc. */
+bool DnetName2PcapName(const char *dnetdev, char *pcapdev, int pcapdevlen);
+
+/* Display the mapping from libdnet interface names (like "eth0") to WinPcap
+   interface names (like "\Device\NPF_{...}"). This is the same mapping used by
+   eth_open and so can help diagnose connection problems.  Additionally display
+   WinPcap interface names that are not mapped to by any libdnet name, in other
+   words the names of interfaces Nmap has no way of using.*/
+static void print_iflist_pcap_mapping(const struct interface_info *iflist, int numifs) {
+  pcap_if_t *pcap_ifs;
+  list<const pcap_if_t *> leftover_pcap_ifs;
+  list<const pcap_if_t *>::iterator leftover_p;
+  int i;
+
+  /* Build a list of "leftover" libpcap interfaces. Initially it contains all
+     the interfaces. */
+  pcap_ifs = getpcapinterfaces();
+  for (const pcap_if_t *p = pcap_ifs; p != NULL; p = p->next)
+    leftover_pcap_ifs.push_front(p);
+
+  if (numifs > 0 || !leftover_pcap_ifs.empty()) {
+    NmapOutputTable Tbl(1 + numifs + leftover_pcap_ifs.size(), 2);
+
+    Tbl.addItem(0, 0, false, "DEV");
+    Tbl.addItem(0, 1, false, "WINDEVICE");
+
+    /* Show the libdnet names and what they map to. */
+    for (i = 0; i < numifs; i++) {
+      char pcap_name[1024];
+
+      if (DnetName2PcapName(iflist[i].devname, pcap_name, sizeof(pcap_name))) {
+        /* We got a name. Remove it from the list of leftovers. */
+        list<const pcap_if_t *>::iterator next;
+        for (leftover_p = leftover_pcap_ifs.begin(); leftover_p != leftover_pcap_ifs.end(); leftover_p = next) {
+          next = leftover_p;
+          next++;
+          if (strcmp((*leftover_p)->name, pcap_name) == 0)
+            leftover_pcap_ifs.erase(leftover_p);
+        }
+      } else {
+        Strncpy(pcap_name, "<none>", sizeof(pcap_name));
+      }
+
+      Tbl.addItem(i + 1, 0, false, iflist[i].devname);
+      Tbl.addItem(i + 1, 1, true, pcap_name);
+    }
+
+    /* Show the "leftover" libpcap interface names (those without a libdnet
+       name that maps to them). */
+    for (leftover_p = leftover_pcap_ifs.begin(); leftover_p != leftover_pcap_ifs.end(); leftover_p++) {
+      Tbl.addItem(i + 1, 0, false, "<none>");
+      Tbl.addItem(i + 1, 1, false, (*leftover_p)->name);
+      i++;
+    }
+
+    log_write(LOG_PLAIN, "%s\n", Tbl.printableTable(NULL));
+    log_flush_all();
+  }
+
+  pcap_freealldevs(pcap_ifs);
+}
+#endif
+
 /* Print a detailed list of Nmap interfaces and routes to
    normal/skiddy/stdout output */
 int print_iflist(void) {
   int numifs = 0, numroutes = 0;
   struct interface_info *iflist;
   struct sys_route *routes;
-  pcap_if_t *p_ifaces, *p_iface_iter;
   NmapOutputTable *Tbl = NULL;
   iflist = getinterfaces(&numifs);
   int i;
@@ -328,25 +398,11 @@ int print_iflist(void) {
     log_flush_all();
     delete Tbl;
   }
-  
-  /* Display windows device names */
-  if((p_ifaces = getpcapinterfaces()) != NULL && numifs > 0) {
-    Tbl = new NmapOutputTable(numifs+1, 2);
-    Tbl->addItem(0, 0, false, "DEV");
-    Tbl->addItem(0, 1, false, "WINDEVICE");
-    i = numifs;
 
-    for(p_iface_iter = p_ifaces; p_iface_iter != NULL && i >= 1; i--) {	
-      Tbl->addItem(i, 0, false, iflist[i-1].devname);
-      Tbl->addItem(i, 1, false, p_iface_iter->name);
-      p_iface_iter = p_iface_iter->next;
-    }
-
-    log_write(LOG_PLAIN, "%s\n", Tbl->printableTable(NULL));
-    log_flush_all();
-    delete Tbl;
-    pcap_freealldevs(p_ifaces);
-  }
+#ifdef WIN32
+  /* Print the libdnet->libpcap interface name mapping. */
+  print_iflist_pcap_mapping(iflist, numifs);
+#endif
 
   /* OK -- time to handle routes */
   routes = getsysroutes(&numroutes);
@@ -417,6 +473,48 @@ static void getNmapServiceName(struct serviceDeductions *sd, int state,
   *namebuf = '\0';  
 }
 
+#ifndef NOLUA
+static char* formatScriptOutput(ScriptResult sr) {
+	std::string result = std::string(), output = sr.get_output();
+	string::size_type pos;
+	char *c_result, *c_output = new char[output.length()+1];
+    strncpy(c_output, output.c_str(), output.length()+1);
+	int line = 0;
+#ifdef WIN32
+	const char* sep = "\r\n";
+#else
+	const char* sep = "\n";
+#endif
+	std::string line_prfx = "|  ";
+
+	char* token = strtok(c_output, sep);
+
+	result += line_prfx + sr.get_id() + ": ";
+
+	while(token != NULL) {
+		if(line > 0)
+			result += line_prfx;
+		result += std::string(token) + sep;
+		token = strtok(NULL, sep);
+		line++;
+	}
+
+	// fix the last line
+	pos = result.rfind(line_prfx);
+	result.replace(pos, 3, "|_ ");
+
+	// delete the unwanted trailing newline
+	pos = result.rfind(sep);
+	if(pos!=std::string::npos){
+		result.erase(pos, strlen(sep));
+	}
+	c_result = strdup(result.c_str());
+
+    delete[] c_output;
+	return c_result;
+}
+#endif /* NOLUA */
+
 /* Prints the familiar Nmap tabular output showing the "interesting"
    ports found on the machine.  It also handles the Machine/Greppable
    output and the XML output.  It is pretty ugly -- in particular I
@@ -426,11 +524,11 @@ void printportoutput(Target *currenths, PortList *plist) {
   char rpcinfo[64];
   char rpcmachineinfo[64];
   char portinfo[64];
-  char xmlbuf[2560];
   char grepvers[256];
   char grepown[64];
   char *p;
-  char *state;
+  char * xmlBuf=NULL;
+  const char *state;
   char serviceinfo[64];
   char *name=NULL;
   int i;
@@ -453,6 +551,9 @@ void printportoutput(Target *currenths, PortList *plist) {
 
   vector<const char *> saved_servicefps;
 
+  if (o.pingscan)
+    return;
+
   log_write(LOG_XML, "<ports>");
   int prevstate = PORT_UNKNOWN;
   int istate;
@@ -496,6 +597,19 @@ void printportoutput(Target *currenths, PortList *plist) {
     return;
   }
 
+  if (o.verbose > 1 || o.debugging) {
+    time_t tm_secs, tm_sece;
+    struct tm *tm;
+    char tbufs[128];
+    tm_secs = currenths->StartTime();    
+    tm_sece = currenths->EndTime();    
+    tm = localtime(&tm_secs);
+	if (strftime(tbufs, sizeof(tbufs), "%Y-%m-%d %H:%M:%S %Z", tm) <= 0)
+      fatal("Unable to properly format host start time");
+
+    log_write(LOG_PLAIN,"Scanned at %s for %lds\n",
+	      tbufs, tm_sece - tm_secs);
+  }
   log_write(LOG_PLAIN,"Interesting %s on %s:\n",
 	    (o.ipprotscan)? "protocols" : "ports", 
 	    currenths->NameIP(hostname, sizeof(hostname)));
@@ -698,9 +812,15 @@ void printportoutput(Target *currenths, PortList *plist) {
 	if (current->owner && *current->owner) {
 	  log_write(LOG_XML, "<owner name=\"%s\" />", current->owner);
 	}
-	if (getServiceXMLBuf(&sd, xmlbuf, sizeof(xmlbuf)) == 0)
-	  if (*xmlbuf)
-	    log_write(LOG_XML, "%s", xmlbuf);
+	
+	if (sd.name || sd.service_fp){
+		xmlBuf = getServiceXMLBuf(&sd);
+		if(xmlBuf){
+			log_write(LOG_XML, "%s", xmlBuf);
+			free(xmlBuf);
+			xmlBuf=NULL;
+		}
+	}
 
 	rowno++;
 #ifndef NOLUA	
@@ -710,8 +830,8 @@ void printportoutput(Target *currenths, PortList *plist) {
 		for(	ssr_iter = current->scriptResults.begin(); 
 			ssr_iter != current->scriptResults.end(); 
 			ssr_iter++) {
-			char* xml_id= xml_convert((*ssr_iter).id);
-			char* xml_scriptoutput= xml_convert((*ssr_iter).output);
+			char* xml_id= xml_convert(ssr_iter->get_id().c_str());
+			char* xml_scriptoutput= xml_convert(ssr_iter->get_output().c_str());
 			log_write(LOG_XML, "<script id=\"%s\" output=\"%s\" />", 
 					xml_id, xml_scriptoutput);
 			free(xml_id);
@@ -732,8 +852,12 @@ void printportoutput(Target *currenths, PortList *plist) {
     
   }
   /*  log_write(LOG_PLAIN,"\n"); */
-  if (plist->getStateCounts(istate) > 0)
-    log_write(LOG_MACHINE, "\tIgnored State: %s (%d)", statenum2str(istate), plist->getStateCounts(istate));
+  /* Grepable output supports only one ignored state. */
+  if (plist->numIgnoredStates() == 1) {
+    istate = plist->nextIgnoredState(PORT_UNKNOWN);
+    if (plist->getStateCounts(istate) > 0)
+      log_write(LOG_MACHINE, "\tIgnored State: %s (%d)", statenum2str(istate), plist->getStateCounts(istate));
+  }
   log_write(LOG_XML, "</ports>\n");
 
   // Now we write the table for the user
@@ -753,47 +877,6 @@ log_write(LOG_PLAIN, "%d service%s unrecognized despite returning data. If you k
   log_flush_all();
 }
 
-#ifndef NOLUA
-char* formatScriptOutput(struct script_scan_result ssr) {
-	char* c_result;
-	std::string result = std::string();
-	string::size_type pos;
-
-	int line = 0;
-#ifdef WIN32
-	char* sep = "\r\n";
-#else
-	char* sep = "\n";
-#endif
-	std::string line_prfx = "|  ";
-	
-	char* token = strtok(ssr.output, sep);
-
-	result += line_prfx + std::string(ssr.id) + ": ";
-
-	while(token != NULL) {
-		if(line > 0)
-			result += line_prfx;
-		result += std::string(token) + sep;
-		token = strtok(NULL, sep);
-		line++;
-	}
-
-	// fix the last line
-	pos = result.rfind(line_prfx);
-	result.replace(pos, 3, "|_ ");
-
-	// delete the unwanted trailing newline
-	pos = result.rfind(sep);
-	if(pos!=std::string::npos){
-		result.erase(pos, strlen(sep));
-	}
-	c_result = strdup(result.c_str());
-
-	return c_result;
-}
-#endif /* NOLUA */
-
 
 /* Note that this escapes newlines, which is generally needed in
    attributes to avoid parser normalization, but might not be needed
@@ -806,7 +889,7 @@ char* xml_convert (const char* str) {
   temp = (char *) safe_malloc(strl*6+1);
   char *end = temp + strl * 6 + 1;
   for (p = temp;(prevch = ch, ch = *str);str++) {
-    char *a;
+    const char *a;
     switch (ch) {
     case '\t':
       a = "&#x9;";
@@ -975,7 +1058,7 @@ void log_vwrite(int logt, const char *fmt, va_list ap) {
 	  /* Windows seems to just give -1 rather than the amount of space we 
 	     would need.  So lets just gulp up a huge amount in the hope it
 	     will be enough */
-	  writebuflen *= 100;
+	  writebuflen *= 150;
 	}
 	writebuf = (char *) safe_realloc(writebuf, writebuflen);
 	len = Vsnprintf(writebuf, writebuflen, fmt, apcopy);
@@ -1145,10 +1228,35 @@ void output_ports_to_machine_parseable_output(struct scan_lists *ports,
  log_flush_all();
 }
 
+//A simple helper function for doscaninfo handles the c14n of o.scanflags
+static void doscanflags(){
+    struct {
+      unsigned char flag;
+      const char *name;
+    } flags[] = {
+      { TH_FIN, "FIN" }, { TH_SYN, "SYN" }, { TH_RST, "RST" },
+      { TH_PUSH, "PSH" }, { TH_ACK, "ACK" }, { TH_URG, "URG" },
+      { TH_ECE, "ECE" }, { TH_CWR, "CWR" }
+    };
+    if(o.scanflags!=-1){
+      log_write(LOG_XML, "scanflags=\"");
+      for(unsigned int i=0;i<sizeof(flags) / sizeof(flags[0]);i++){
+        if(o.scanflags & flags[i].flag){
+          log_write(LOG_XML, "%s",flags[i].name);
+        }
+      }
+      log_write(LOG_XML, "\"");
+    }
+}
+
 /* Simple helper function for output_xml_scaninfo_records */
-static void doscaninfo(char *type, char *proto, unsigned short *ports, 
+static void doscaninfo(const char *type, const char *proto, unsigned short *ports, 
 		  int numports) {
-  log_write(LOG_XML, "<scaninfo type=\"%s\" protocol=\"%s\" numservices=\"%d\" services=\"", type, proto, numports);
+  log_write(LOG_XML, "<scaninfo type=\"%s\" ", type);
+  if(strncmp(proto,"tcp",3)==0){
+	doscanflags();
+  }
+  log_write(LOG_XML, " protocol=\"%s\" numservices=\"%d\" services=\"", proto, numports);
   output_rangelist_given_ports(LOG_XML, ports, numports);
   log_write(LOG_XML, "\" />\n");
 }
@@ -1359,9 +1467,9 @@ static void printosclassificationoutput(const struct OS_Classification_Results *
     // Now to create the fodder for normal output
     for (classno=0; classno < OSR->OSC_num_matches; classno++) {
       /* We have processed enough if any of the following are true */
-      if (!guess && OSR->OSC_Accuracy[classno] < 1.0 ||
+      if ((!guess && OSR->OSC_Accuracy[classno] < 1.0) ||
 	  OSR->OSC_Accuracy[classno] <= OSR->OSC_Accuracy[0] - 0.1 ||
-	  OSR->OSC_Accuracy[classno] < 1.0 && classno > 9)
+	  (OSR->OSC_Accuracy[classno] < 1.0 && classno > 9))
 	break;
       if (addtochararrayifnew(types, &numtypes, MAX_OS_CLASSMEMBERS, OSR->OSC[classno]->Device_Type) == -1)
 	overflow = 1;
@@ -1452,44 +1560,15 @@ void printosscanoutput(Target *currenths) {
   char numlst[512]; /* For creating lists of numbers */
   char *p; /* Used in manipulating numlst above */
   FingerPrintResults *FPR;
-  int osscanSys = 0;
   int distance = -1;
   int osscan_flag;
   
   if (!(osscan_flag = currenths->osscanPerformed()))
     return;
    
-  if (currenths->FPR == NULL && currenths->FPR1 == NULL) {
+  if (currenths->FPR == NULL)
     return;
-  } else if (currenths->FPR != NULL && currenths->FPR1 == NULL) {
-    osscanSys = 2;
-    FPR = currenths->FPR;
-  } else if (currenths->FPR == NULL && currenths->FPR1 != NULL) {
-    osscanSys = 1;
-    FPR = currenths->FPR1;
-  }
-  else {
-    /* Neither is NULL. This happens when new OS scan system fails to
-       get a perfect match and falls back on the old OS scan
-       system. */
-    if (currenths->FPR->num_perfect_matches > 0) {
-      osscanSys = 2;
-      FPR = currenths->FPR; /* Just an ensurance. */
-    } else if (currenths->FPR1->num_perfect_matches > 0) {
-      osscanSys = 1;
-      FPR = currenths->FPR1;
-    } else if (currenths->FPR->overall_results == OSSCAN_SUCCESS) {
-      osscanSys = 2;
-      FPR = currenths->FPR;
-    } else if (currenths->FPR1->overall_results == OSSCAN_SUCCESS) {
-      osscanSys = 1;
-      FPR = currenths->FPR1;
-    } else {
-      /* Both fails. */
-      osscanSys = 2;
-      FPR = currenths->FPR;
-    }
-  }
+  FPR = currenths->FPR;
 
   if (currenths->distance != -1)
 	distance = currenths->distance;
@@ -1581,7 +1660,7 @@ void printosscanoutput(Target *currenths) {
 	}
 	log_write(LOG_PLAIN, "\n");
       }
-      if (osscanSys == 2 && !reason) {
+      if (!reason) {
 	log_write(LOG_NORMAL|LOG_SKID_NOXLT|LOG_STDOUT,"No exact OS matches for host (If you know what OS is running on it, see http://nmap.org/submit/ ).\nTCP/IP fingerprint:\n%s\n",
 		  mergeFPs(FPR->FPs, FPR->numFPs, true,
 			   currenths->v4hostip(), distance, currenths->MACAddress(),
@@ -1593,8 +1672,8 @@ void printosscanoutput(Target *currenths) {
 	  log_write(LOG_NORMAL|LOG_SKID_NOXLT|LOG_STDOUT,"No exact OS matches for host (test conditions non-ideal).");
 	  if (o.verbose > 1 || o.debugging)
 	    log_write(LOG_NORMAL|LOG_SKID_NOXLT|LOG_STDOUT, 
-		      "\nTCP/IP fingerprint by osscan system #%d:\n%s",
-		      osscanSys, mergeFPs(FPR->FPs, FPR->numFPs, false,
+		      "\nTCP/IP fingerprint:\n%s",
+		      mergeFPs(FPR->FPs, FPR->numFPs, false,
 					  currenths->v4hostip(), distance, currenths->MACAddress(),
 					  FPR->osscan_opentcpport, FPR->osscan_closedtcpport, FPR->osscan_closedudpport,
 					  false));
@@ -1616,7 +1695,7 @@ void printosscanoutput(Target *currenths) {
     const char *reason = FPR->OmitSubmissionFP();
     if ((o.verbose > 1 || o.debugging) && reason)
       log_write(LOG_NORMAL|LOG_SKID_NOXLT|LOG_STDOUT,"OS fingerprint not ideal because: %s\n", reason);
-    if (osscanSys == 2 && !reason) {
+    if (!reason) {
       log_write(LOG_NORMAL|LOG_SKID_NOXLT|LOG_STDOUT,"No OS matches for host (If you know what OS is running on it, see http://nmap.org/submit/ ).\nTCP/IP fingerprint:\n%s\n",
 		mergeFPs(FPR->FPs, FPR->numFPs, true,
 			 currenths->v4hostip(), distance, currenths->MACAddress(),
@@ -1625,8 +1704,8 @@ void printosscanoutput(Target *currenths) {
     } else {
       log_write(LOG_NORMAL|LOG_SKID_NOXLT|LOG_STDOUT,"No OS matches for host\n");
       if (o.verbose > 1)
-	log_write(LOG_NORMAL|LOG_SKID_NOXLT|LOG_STDOUT, "\nTCP/IP fingerprint by osscan system #%d:\n%s",
-		  osscanSys, mergeFPs(FPR->FPs, FPR->numFPs, false,
+	log_write(LOG_NORMAL|LOG_SKID_NOXLT|LOG_STDOUT, "\nTCP/IP fingerprint:\n%s",
+		  mergeFPs(FPR->FPs, FPR->numFPs, false,
 				      currenths->v4hostip(), distance, currenths->MACAddress(),
 				      FPR->osscan_opentcpport, FPR->osscan_closedtcpport, FPR->osscan_closedudpport,
 				      false));
@@ -1634,8 +1713,8 @@ void printosscanoutput(Target *currenths) {
   } else if (FPR->overall_results == OSSCAN_TOOMANYMATCHES || (FPR->num_perfect_matches > 8 && !o.debugging)) {
     log_write(LOG_PLAIN,"Too many fingerprints match this host to give specific OS details\n");
     if (o.debugging || o.verbose > 1) {
-      log_write(LOG_PLAIN,"TCP/IP fingerprint by osscan system #%d:\n%s",
-		osscanSys, mergeFPs(FPR->FPs, FPR->numFPs, false,
+      log_write(LOG_PLAIN,"TCP/IP fingerprint:\n%s",
+		mergeFPs(FPR->FPs, FPR->numFPs, false,
 				    currenths->v4hostip(), distance, currenths->MACAddress(),
 				    FPR->osscan_opentcpport, FPR->osscan_closedtcpport, FPR->osscan_closedudpport,
 				    false));
@@ -1659,7 +1738,8 @@ void printosscanoutput(Target *currenths) {
     gettimeofday(&tv, NULL);
     strncpy(tmbuf, ctime(&(currenths->seq.lastboot)), sizeof(tmbuf));
     chomp(tmbuf);
-    log_write(LOG_PLAIN,"Uptime: %.3f days (since %s)\n", (double) (tv.tv_sec - currenths->seq.lastboot) / 86400, tmbuf);
+    if (o.verbose)
+      log_write(LOG_PLAIN,"Uptime guess: %.3f days (since %s)\n", (double) (tv.tv_sec - currenths->seq.lastboot) / 86400, tmbuf);
     log_write(LOG_XML, "<uptime seconds=\"%li\" lastboot=\"%s\" />\n", tv.tv_sec - currenths->seq.lastboot, tmbuf);
   }
   
@@ -1678,13 +1758,9 @@ void printosscanoutput(Target *currenths) {
       while(*p) p++;
     }
     
-    log_write(LOG_XML, "<tcpsequence index=\"%li\" class=\"%s\" difficulty=\"%s\" values=\"%s\" />\n", (long) currenths->seq.index, seqclass2ascii(currenths->seq.seqclass), seqidx2difficultystr(currenths->seq.index), numlst); 
-    if (o.verbose) {
-      if (osscanSys == 1)
-	log_write(LOG_PLAIN,"%s", seqreport1(&(currenths->seq)));
-      else if(osscanSys == 2)
-	log_write(LOG_PLAIN,"%s", seqreport(&(currenths->seq)));
-    }
+    log_write(LOG_XML, "<tcpsequence index=\"%li\" difficulty=\"%s\" values=\"%s\" />\n", (long) currenths->seq.index, seqidx2difficultystr(currenths->seq.index), numlst); 
+    if (o.verbose)
+      log_write(LOG_PLAIN,"%s", seqreport(&(currenths->seq)));
     
     log_write(LOG_MACHINE,"\tSeq Index: %d", currenths->seq.index);
   }
@@ -1738,7 +1814,7 @@ void printserviceinfooutput(Target *currenths) {
   char hostname_tbl[MAX_SERVICE_INFO_FIELDS][MAXHOSTNAMELEN];
   char ostype_tbl[MAX_SERVICE_INFO_FIELDS][64];
   char devicetype_tbl[MAX_SERVICE_INFO_FIELDS][64];
-  char *delim;
+  const char *delim;
 
   for (i=0; i<MAX_SERVICE_INFO_FIELDS; i++)
     hostname_tbl[i][0] = ostype_tbl[i][0] = devicetype_tbl[i][0] = '\0';
@@ -1833,8 +1909,8 @@ void printhostscriptresults(Target *currenths) {
 		log_write(LOG_XML, "<hostscript>");
 		log_write(LOG_PLAIN, "\nHost script results:\n");
 		for(iter = currenths->scriptResults.begin(); iter != currenths->scriptResults.end(); iter++) {
-			xml_id = xml_convert((*iter).id);
-			xml_scriptoutput= xml_convert((*iter).output);
+			xml_id = xml_convert(iter->get_id().c_str());
+			xml_scriptoutput= xml_convert(iter->get_output().c_str());
 			log_write(LOG_XML, "<script id=\"%s\" output=\"%s\" />", 
 					xml_id, xml_scriptoutput);
 			script_output = formatScriptOutput((*iter));
@@ -1857,7 +1933,7 @@ void printStatusMessage() {
   
   log_write(LOG_STDOUT, 
 	    "Stats: %d:%02d:%02d elapsed; %d hosts completed (%d up), %d undergoing %s\n", 
-	    time/60/24, time/60 % 24, time % 60, o.numhosts_scanned - o.numhosts_scanning, 
+	    time/60/60, time/60 % 60, time % 60, o.numhosts_scanned - o.numhosts_scanning, 
 	    o.numhosts_up, o.numhosts_scanning, scantype2str(o.current_scantype));
 }
 
@@ -1891,7 +1967,7 @@ void printfinaloutput() {
       log_write(LOG_PLAIN, "Service detection performed. Please report any incorrect results at http://nmap.org/submit/ .\n");
   }
 
-  log_write(LOG_STDOUT|LOG_SKID, "Nmap done: %d %s (%d %s up) scanned in %.3f seconds\n", o.numhosts_scanned, (o.numhosts_scanned == 1)? "IP address" : "IP addresses", o.numhosts_up, (o.numhosts_up == 1)? "host" : "hosts",  o.TimeSinceStartMS(&tv) / 1000.0);
+  log_write(LOG_STDOUT|LOG_SKID, "Nmap done: %d %s (%d %s up) scanned in %.2f seconds\n", o.numhosts_scanned, (o.numhosts_scanned == 1)? "IP address" : "IP addresses", o.numhosts_up, (o.numhosts_up == 1)? "host" : "hosts",  o.TimeSinceStartMS(&tv) / 1000.0);
   if (o.verbose && o.isr00t && o.RawScan()) 
     log_write(LOG_STDOUT|LOG_SKID, "           %s\n", 
 	      getFinalPacketStats(statbuf, sizeof(statbuf)));
@@ -1901,8 +1977,8 @@ void printfinaloutput() {
   
   log_write(LOG_XML, "<runstats><finished time=\"%lu\" timestr=\"%s\"/><hosts up=\"%d\" down=\"%d\" total=\"%d\" />\n", (unsigned long) timep, mytime, o.numhosts_up, o.numhosts_scanned - o.numhosts_up, o.numhosts_scanned);
 
-  log_write(LOG_XML, "<!-- Nmap done at %s; %d %s (%d %s up) scanned in %.3f seconds -->\n", mytime, o.numhosts_scanned, (o.numhosts_scanned == 1)? "IP address" : "IP addresses", o.numhosts_up, (o.numhosts_up == 1)? "host" : "hosts",  o.TimeSinceStartMS(&tv) / 1000.0 );
-  log_write(LOG_NORMAL|LOG_MACHINE, "# Nmap done at %s -- %d %s (%d %s up) scanned in %.3f seconds\n", mytime, o.numhosts_scanned, (o.numhosts_scanned == 1)? "IP address" : "IP addresses", o.numhosts_up, (o.numhosts_up == 1)? "host" : "hosts", o.TimeSinceStartMS(&tv) / 1000.0 );
+  log_write(LOG_XML, "<!-- Nmap done at %s; %d %s (%d %s up) scanned in %.2f seconds -->\n", mytime, o.numhosts_scanned, (o.numhosts_scanned == 1)? "IP address" : "IP addresses", o.numhosts_up, (o.numhosts_up == 1)? "host" : "hosts",  o.TimeSinceStartMS(&tv) / 1000.0 );
+  log_write(LOG_NORMAL|LOG_MACHINE, "# Nmap done at %s -- %d %s (%d %s up) scanned in %.2f seconds\n", mytime, o.numhosts_scanned, (o.numhosts_scanned == 1)? "IP address" : "IP addresses", o.numhosts_up, (o.numhosts_up == 1)? "host" : "hosts", o.TimeSinceStartMS(&tv) / 1000.0 );
 
   log_write(LOG_XML, "</runstats></nmaprun>\n");
   log_flush_all();
@@ -1990,7 +2066,7 @@ void printdatafilepaths() {
     /* If all the files were from the same directory and we're in verbose mode,
        print a brief message unless we are also in debugging mode. */
     log_write(LOG_PLAIN, "Read data files from: %s\n", dir.c_str());
-  } else if (num_dirs == 1 && o.debugging || num_dirs > 1) {
+  } else if ( (num_dirs == 1 && o.debugging) || num_dirs > 1) {
     /* If files were read from more than one directory, or if they were read
        from one directory and we are in debugging mode, display all the files
        grouped by directory. */
