@@ -25,7 +25,7 @@
  * following:                                                              *
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
- *   nmap-os-fingerprints or nmap-service-probes.                          *
+ *   nmap-os-db or nmap-service-probes.                                    *
  * o Executes Nmap and parses the results (as opposed to typical shell or  *
  *   execution-menu apps, which simply display raw Nmap output and so are  *
  *   not derivative works.)                                                * 
@@ -60,7 +60,7 @@
  * As a special exception to the GPL terms, Insecure.Com LLC grants        *
  * permission to link the code of this program with any version of the     *
  * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * listed in the included COPYING.OpenSSL file, and distribute linked      *
  * combinations including the two. You must obey the GNU GPL in all        *
  * respects for all of the code used other than OpenSSL.  If you modify    *
  * this file, you may extend this exception to your version of the file,   *
@@ -92,13 +92,13 @@
  * This program is distributed in the hope that it will be useful, but     *
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
- * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
- * with Nmap.                                                              *
+ * General Public License v2.0 for more details at                         *
+ * http://www.gnu.org/licenses/gpl-2.0.html , or in the COPYING file       *
+ * included with Nmap.                                                     *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: targets.cc 6858 2008-02-28 18:52:06Z fyodor $ */
+/* $Id: targets.cc 9176 2008-07-29 17:01:31Z michael $ */
 
 
 #include "targets.h"
@@ -130,7 +130,7 @@ static inline int gethostnum(Target *hostbatch[], Target *target) {
   return 0; // Unreached
 }
 
-char *readhoststate(int state) {
+const char *readhoststate(int state) {
   switch(state) {
   case HOST_UP:
     return "HOST_UP";
@@ -228,7 +228,7 @@ static int hostInExclude(struct sockaddr *checksock, size_t checksocklen,
       /* For Netmasks simply compare the network bits and move to the next
        * group if it does not compare, we don't care about the individual addrs */
       if (targets_type == TargetGroup::IPV4_NETMASK) {
-        mask = htonl((unsigned long) (0-1) << 32-exclude_group[i].get_mask());
+        mask = htonl((unsigned long) (0-1) << (32-exclude_group[i].get_mask()));
         if ((tmpTarget & mask) == (checkhost->sin_addr.s_addr & mask)) {
 	  exclude_group[i].rewind();
 	  return 1;
@@ -429,7 +429,7 @@ int dumpExclude(TargetGroup *exclude_group) {
   return 1;
 }
  
-static void massping(Target *hostbatch[], int num_hosts, int pingtype) {
+static void massping(Target *hostbatch[], int num_hosts, struct scan_lists *ports) {
   static struct timeout_info group_to = { 0, 0, 0 };
   static char prev_device_name[16] = "";
   const char *device_name;
@@ -459,8 +459,7 @@ static void massping(Target *hostbatch[], int num_hosts, int pingtype) {
     targets.push_back(hostbatch[i]);
   }
 
-  /* ultra_scan gets pingtype from o.pingtype. */
-  ultra_scan(targets, NULL, PING_SCAN, &group_to);
+  ultra_scan(targets, ports, PING_SCAN, &group_to);
 }
 
 Target *nexthost(HostGroupState *hs, TargetGroup *exclude_group,
@@ -607,14 +606,15 @@ if (hs->randomize) {
      if (!hs->hostbatch[i]->timedOut(&now)) {
        initialize_timeout_info(&hs->hostbatch[i]->to);
        hs->hostbatch[i]->flags |= HOST_UP; /*hostbatch[i].up = 1;*/
-	   hs->hostbatch[i]->reason.reason_id = ER_LOCALHOST;
+       if(pingtype == PINGTYPE_NONE && !arpping_done)
+         hs->hostbatch[i]->reason.reason_id = ER_USER;
+       else
+         hs->hostbatch[i]->reason.reason_id = ER_LOCALHOST;
      }
    }
- } else if (!arpping_done)
-   if (pingtype & PINGTYPE_ARP) /* A host that we can't arp scan ... maybe localhost */
-     massping(hs->hostbatch, hs->current_batch_sz, DEFAULT_PING_TYPES);
-   else
-     massping(hs->hostbatch, hs->current_batch_sz, pingtype);
+ } else if (!arpping_done) {
+   massping(hs->hostbatch, hs->current_batch_sz, ports);
+ }
  
  if (!o.noresolve) nmap_mass_rdns(hs->hostbatch, hs->current_batch_sz);
  
